@@ -1,25 +1,34 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, cast
+
 import numpy as np
 from numpy.testing import assert_allclose, assert_array_equal
 import pytest
 
-from contourpy import LineType, contour_generator
+from contourpy import LineType, contour_generator, max_threads
 from contourpy.util.data import random, simple
 
 from . import util_test
 
-
-@pytest.fixture
-def xy_2x2():
-    return np.meshgrid([0.0, 1.0], [0.0, 1.0])
+if TYPE_CHECKING:
+    import contourpy._contourpy as cpy
 
 
 @pytest.fixture
-def xy_3x3():
-    return np.meshgrid([0.0, 0.5, 1.0], [0.0, 0.5, 1.0])
+def xy_2x2() -> tuple[cpy.CoordinateArray, ...]:
+    x, y = np.meshgrid([0.0, 1.0], [0.0, 1.0])
+    return x, y
 
 
 @pytest.fixture
-def one_loop_one_strip():
+def xy_3x3() -> tuple[cpy.CoordinateArray, ...]:
+    x, y = np.meshgrid([0.0, 0.5, 1.0], [0.0, 0.5, 1.0])
+    return x, y
+
+
+@pytest.fixture
+def one_loop_one_strip() -> tuple[cpy.PointArray, ...]:
     x, y = np.meshgrid([0., 1., 2., 3.], [0., 1., 2.])
     z = np.array([[1.5, 1.5, 0.9, 0.0],
                   [1.5, 2.8, 0.4, 0.8],
@@ -28,7 +37,7 @@ def one_loop_one_strip():
 
 
 @pytest.fixture
-def xyz_chunk_test():
+def xyz_chunk_test() -> tuple[cpy.CoordinateArray, ...]:
     x, y = np.meshgrid(np.arange(5), np.arange(5))
     z = 0.5*np.abs(y - 2) + 0.1*(x - 2)
     return x, y, z
@@ -36,11 +45,13 @@ def xyz_chunk_test():
 
 @pytest.mark.parametrize("name", util_test.all_names())
 @pytest.mark.parametrize("zlevel", [-1e-10, 1.0+1e10, np.nan, -np.nan, np.inf, -np.inf])
-def test_level_outside(xy_2x2, name, zlevel):
+def test_level_outside(xy_2x2: tuple[cpy.CoordinateArray, ...], name: str, zlevel: float) -> None:
     x, y = xy_2x2
     z = x
     cont_gen = contour_generator(x, y, z, name=name, line_type=LineType.SeparateCode)
     lines = cont_gen.lines(zlevel)
+    if TYPE_CHECKING:
+        lines = cast(cpy.LineReturn_SeparateCode, lines)
     assert isinstance(lines, tuple) and len(lines) == 2
     points, codes = lines
     assert isinstance(points, list) and len(points) == 0
@@ -48,11 +59,13 @@ def test_level_outside(xy_2x2, name, zlevel):
 
 
 @pytest.mark.parametrize("name", util_test.all_names())
-def test_w_to_e(xy_2x2, name):
+def test_w_to_e(xy_2x2: tuple[cpy.CoordinateArray, ...], name: str) -> None:
     x, y = xy_2x2
     z = y.copy()
     cont_gen = contour_generator(x, y, z, name=name, line_type=LineType.SeparateCode)
     lines = cont_gen.lines(0.5)
+    if TYPE_CHECKING:
+        lines = cast(cpy.LineReturn_SeparateCode, lines)
     assert isinstance(lines, tuple) and len(lines) == 2
     points, codes = lines
     assert isinstance(points, list) and len(points) == 1
@@ -62,11 +75,13 @@ def test_w_to_e(xy_2x2, name):
 
 
 @pytest.mark.parametrize("name", util_test.all_names())
-def test_e_to_w(xy_2x2, name):
+def test_e_to_w(xy_2x2: tuple[cpy.CoordinateArray, ...], name: str) -> None:
     x, y = xy_2x2
     z = 1.0 - y.copy()
     cont_gen = contour_generator(x, y, z, name=name, line_type=LineType.SeparateCode)
     lines = cont_gen.lines(0.5)
+    if TYPE_CHECKING:
+        lines = cast(cpy.LineReturn_SeparateCode, lines)
     assert isinstance(lines, tuple) and len(lines) == 2
     points, codes = lines
     assert isinstance(points, list) and len(points) == 1
@@ -79,12 +94,14 @@ def test_e_to_w(xy_2x2, name):
 
 
 @pytest.mark.parametrize("name", util_test.all_names())
-def test_loop(xy_3x3, name):
+def test_loop(xy_3x3: tuple[cpy.CoordinateArray, ...], name: str) -> None:
     x, y = xy_3x3
     z = np.zeros_like(x)
     z[1, 1] = 1.0
     cont_gen = contour_generator(x, y, z, name=name, line_type=LineType.SeparateCode)
     lines = cont_gen.lines(0.5)
+    if TYPE_CHECKING:
+        lines = cast(cpy.LineReturn_SeparateCode, lines)
     assert isinstance(lines, tuple) and len(lines) == 2
     points, codes = lines
     assert isinstance(points, list) and len(points) == 1
@@ -97,7 +114,7 @@ def test_loop(xy_3x3, name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_simple(name, line_type):
+def test_lines_simple(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -120,7 +137,7 @@ def test_lines_simple(name, line_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_simple_chunk(name, line_type):
+def test_lines_simple_chunk(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -147,7 +164,7 @@ def test_lines_simple_chunk(name, line_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_simple_chunk_threads(line_type, thread_count):
+def test_lines_simple_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -172,7 +189,7 @@ def test_lines_simple_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_simple_no_corner_mask(name, line_type):
+def test_lines_simple_no_corner_mask(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -195,7 +212,7 @@ def test_lines_simple_no_corner_mask(name, line_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_simple_no_corner_mask_chunk(name, line_type):
+def test_lines_simple_no_corner_mask_chunk(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -222,7 +239,7 @@ def test_lines_simple_no_corner_mask_chunk(name, line_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_simple_no_corner_mask_chunk_threads(line_type, thread_count):
+def test_lines_simple_no_corner_mask_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -250,7 +267,7 @@ def test_lines_simple_no_corner_mask_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_lines_simple_corner_mask(name):
+def test_lines_simple_corner_mask(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -274,7 +291,7 @@ def test_lines_simple_corner_mask(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_lines_simple_corner_mask_chunk(name):
+def test_lines_simple_corner_mask_chunk(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -302,7 +319,7 @@ def test_lines_simple_corner_mask_chunk(name):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_simple_corner_mask_chunk_threads(line_type, thread_count):
+def test_lines_simple_corner_mask_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -330,7 +347,7 @@ def test_lines_simple_corner_mask_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.quad_as_tri_names())
-def test_lines_simple_quad_as_tri(name):
+def test_lines_simple_quad_as_tri(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -354,7 +371,7 @@ def test_lines_simple_quad_as_tri(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_random(name, line_type):
+def test_lines_random(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -377,7 +394,7 @@ def test_lines_random(name, line_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_random_chunk(name, line_type):
+def test_lines_random_chunk(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -399,7 +416,7 @@ def test_lines_random_chunk(name, line_type):
 
     max_threshold = None
     mean_threshold = None
-    if name == 'mpl2005':
+    if name == "mpl2005":
         max_threshold = 126
         mean_threshold = 0.11
 
@@ -413,7 +430,7 @@ def test_lines_random_chunk(name, line_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_random_chunk_threads(line_type, thread_count):
+def test_lines_random_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -440,7 +457,7 @@ def test_lines_random_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_random_no_corner_mask(name, line_type):
+def test_lines_random_no_corner_mask(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -463,7 +480,7 @@ def test_lines_random_no_corner_mask(name, line_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
-def test_lines_random_no_corner_mask_chunk(name, line_type):
+def test_lines_random_no_corner_mask_chunk(name: str, line_type: LineType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -493,7 +510,7 @@ def test_lines_random_no_corner_mask_chunk(name, line_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_random_no_corner_mask_chunk_threads(line_type, thread_count):
+def test_lines_random_no_corner_mask_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -521,7 +538,7 @@ def test_lines_random_no_corner_mask_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_lines_random_corner_mask(name):
+def test_lines_random_corner_mask(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -545,7 +562,7 @@ def test_lines_random_corner_mask(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_lines_random_corner_mask_chunk(name):
+def test_lines_random_corner_mask_chunk(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -573,7 +590,7 @@ def test_lines_random_corner_mask_chunk(name):
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_lines_random_corner_mask_chunk_threads(line_type, thread_count):
+def test_lines_random_corner_mask_chunk_threads(line_type: LineType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -601,7 +618,7 @@ def test_lines_random_corner_mask_chunk_threads(line_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.quad_as_tri_names())
-def test_lines_random_quad_as_tri(name):
+def test_lines_random_quad_as_tri(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -625,7 +642,11 @@ def test_lines_random_quad_as_tri(name):
 
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("name", ["serial", "threaded"])
-def test_return_by_line_type(one_loop_one_strip, name, line_type):
+def test_return_by_line_type(
+    one_loop_one_strip: tuple[cpy.PointArray, ...],
+    name: str,
+    line_type: LineType,
+) -> None:
     x, y, z = one_loop_one_strip
     cont_gen = contour_generator(x, y, z, name=name, line_type=line_type)
 
@@ -638,11 +659,15 @@ def test_return_by_line_type(one_loop_one_strip, name, line_type):
     util_test.assert_lines(lines, line_type)
 
     if line_type == LineType.Separate:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_Separate, lines)
         points = lines
         assert len(points) == 2
         assert points[0].shape == (5, 2)
         assert points[1].shape == (2, 2)
     elif line_type == LineType.SeparateCode:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_SeparateCode, lines)
         points, codes = lines
         assert len(points) == 2
         assert points[0].shape == (5, 2)
@@ -650,22 +675,48 @@ def test_return_by_line_type(one_loop_one_strip, name, line_type):
         assert_array_equal(codes[0], [1, 2, 2, 2, 79])
         assert_array_equal(codes[1], [1, 2])
     elif line_type == LineType.ChunkCombinedCode:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedCode, lines)
         assert len(lines[0]) == 1  # Single chunk.
-        points, codes = lines[0][0], lines[1][0]
-        assert points.shape == (7, 2)
-        assert_array_equal(codes, [1, 2, 2, 2, 79, 1, 2])
+        points_or_none, codes_or_none = lines[0][0], lines[1][0]
+        assert points_or_none is not None
+        assert codes_or_none is not None
+        assert points_or_none.shape == (7, 2)
+        assert_array_equal(codes_or_none, [1, 2, 2, 2, 79, 1, 2])
     elif line_type == LineType.ChunkCombinedOffset:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedOffset, lines)
         assert len(lines[0]) == 1  # Single chunk.
-        points, offsets = lines[0][0], lines[1][0]
-        assert points.shape == (7, 2)
-        assert_array_equal(offsets, [0, 5, 7])
+        points_or_none, offsets_or_none = lines[0][0], lines[1][0]
+        assert points_or_none is not None
+        assert offsets_or_none is not None
+        assert points_or_none.shape == (7, 2)
+        assert_array_equal(offsets_or_none, [0, 5, 7])
+    elif line_type == LineType.ChunkCombinedNan:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedNan, lines)
+        assert len(lines[0]) == 1  # Single chunk
+        points_or_none = lines[0][0]
+        assert points_or_none is not None
+        assert points_or_none.shape == (8, 2)
+        assert np.all(np.isnan(points_or_none[5, :]))
+    else:
+        raise RuntimeError(f"Unexpected line_type {line_type}")
 
 
 @pytest.mark.threads
 @pytest.mark.parametrize("line_type", LineType.__members__.values())
 @pytest.mark.parametrize("name, thread_count",
-                         [("serial", 1), ("threaded", 1), ("threaded", 2)])
-def test_return_by_line_type_chunk(xyz_chunk_test, name, thread_count, line_type):
+                         [("serial", 1), ("threaded", 1),
+                         pytest.param("threaded", 2,
+                            marks = pytest.mark.skipif(
+                            max_threads() <= 1, reason = "executing on unicore host"))])
+def test_return_by_line_type_chunk(
+    xyz_chunk_test: tuple[cpy.CoordinateArray, ...],
+    name: str,
+    thread_count: int,
+    line_type: LineType,
+) -> None:
     x, y, z = xyz_chunk_test
     cont_gen = contour_generator(
         x, y, z, name=name, line_type=line_type, chunk_count=2, thread_count=thread_count,
@@ -681,14 +732,16 @@ def test_return_by_line_type_chunk(xyz_chunk_test, name, thread_count, line_type
     util_test.assert_lines(lines, line_type)
 
     # Expected points by chunk.
-    expected = (
-        [[2.0, 1.1], [1.5, 1.0], [1.0, 0.9], [0.0, 0.7]],
-        [[4.0, 1.5], [3.0, 1.3], [2.0, 1.1]],
-        [[0.0, 3.3], [1.0, 3.1], [1.5, 3.0], [2.0, 2.9]],
-        [[2.0, 2.9], [3.0, 2.7], [4.0, 2.5]],
-    )
+    expected: list[cpy.PointArray] = [
+        np.asarray([[2.0, 1.1], [1.5, 1.0], [1.0, 0.9], [0.0, 0.7]]),
+        np.asarray([[4.0, 1.5], [3.0, 1.3], [2.0, 1.1]]),
+        np.asarray([[0.0, 3.3], [1.0, 3.1], [1.5, 3.0], [2.0, 2.9]]),
+        np.asarray([[2.0, 2.9], [3.0, 2.7], [4.0, 2.5]]),
+    ]
 
     if line_type == LineType.Separate:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_Separate, lines)
         assert len(lines) == 4
         if name == "threaded" and cont_gen.thread_count > 1:
             # Lines may be in any order so sort lines and expected.
@@ -697,6 +750,8 @@ def test_return_by_line_type_chunk(xyz_chunk_test, name, thread_count, line_type
         for chunk in range(4):
             assert_allclose(lines[chunk], expected[chunk])
     elif line_type == LineType.SeparateCode:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_SeparateCode, lines)
         assert len(lines[0]) == 4
         if name == "threaded" and cont_gen.thread_count > 1:
             # Lines may be in any order so sort lines and expected.
@@ -706,20 +761,41 @@ def test_return_by_line_type_chunk(xyz_chunk_test, name, thread_count, line_type
             assert_allclose(lines[0][chunk], expected[chunk])
             assert_array_equal(lines[1][chunk], [1] + [2]*(len(expected[chunk])-1))
     elif line_type == LineType.ChunkCombinedCode:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedCode, lines)
         assert len(lines[0]) == 4
         for chunk in range(4):
-            assert_allclose(lines[0][chunk], expected[chunk])
-            assert_array_equal(lines[1][chunk], [1] + [2]*(len(expected[chunk])-1))
+            points_or_none, codes_or_none = lines[0][chunk], lines[1][chunk]
+            assert points_or_none is not None
+            assert codes_or_none is not None
+            assert_allclose(points_or_none, expected[chunk])
+            assert_array_equal(codes_or_none, [1] + [2]*(len(expected[chunk])-1))
     elif line_type == LineType.ChunkCombinedOffset:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedOffset, lines)
         assert len(lines[0]) == 4
         for chunk in range(4):
-            assert_allclose(lines[0][chunk], expected[chunk])
-            assert_array_equal(lines[1][chunk], [0, len(expected[chunk])])
+            points_or_none, offsets_or_none = lines[0][chunk], lines[1][chunk]
+            assert points_or_none is not None
+            assert offsets_or_none is not None
+            assert_allclose(points_or_none, expected[chunk])
+            assert_array_equal(offsets_or_none, [0, len(expected[chunk])])
+    elif line_type == LineType.ChunkCombinedNan:
+        if TYPE_CHECKING:
+            lines = cast(cpy.LineReturn_ChunkCombinedNan, lines)
+        assert len(lines[0]) == 4
+        for chunk in range(4):
+            # Only a single line per chunk, so no nans here.
+            points_or_none = lines[0][chunk]
+            assert points_or_none is not None
+            assert_allclose(points_or_none, expected[chunk])
+    else:
+        raise RuntimeError(f"Unexpected line_type {line_type}")
 
 
 @pytest.mark.parametrize("name, line_type", util_test.all_names_and_line_types())
 @pytest.mark.parametrize("corner_mask", [None, False, True])
-def test_lines_random_big(name, line_type, corner_mask):
+def test_lines_random_big(name: str, line_type: LineType, corner_mask: bool) -> None:
     if corner_mask and name in ["mpl2005", "mpl2014"]:
         pytest.skip()
 
@@ -734,3 +810,20 @@ def test_lines_random_big(name, line_type, corner_mask):
     for level in levels:
         lines = cont_gen.lines(level)
         util_test.assert_lines(lines, line_type)
+
+
+@pytest.mark.parametrize("name", util_test.all_names())
+@pytest.mark.parametrize("z", [np.nan, -np.nan, np.inf, -np.inf])
+@pytest.mark.parametrize("zlevel", [0.0, np.nan, -np.nan, np.inf, -np.inf])
+def test_lines_z_nonfinite(name: str, z: float, zlevel: float) -> None:
+    cont_gen = contour_generator(z=[[z, z], [z, z]], name=name, line_type=LineType.SeparateCode)
+    lines = cont_gen.lines(zlevel)
+    assert lines == ([], [])
+
+
+@pytest.mark.parametrize("name", util_test.all_names())
+def test_lines_infinite_level(name: str) -> None:
+    cont_gen = contour_generator(z=[[0, 0], [1, 1]], name=name, line_type=LineType.SeparateCode)
+    for level in (-np.inf, np.inf):
+        lines = cont_gen.lines(level)
+        assert len(lines[0]) == 0  # No lines
